//
//
//****************************************************************************************//
// Copyright (c) 2002-2011, The MITRE Corporation
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright notice, this list
//       of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright notice, this 
//       list of conditions and the following disclaimer in the documentation and/or other
//       materials provided with the distribution.
//     * Neither the name of The MITRE Corporation nor the names of its contributors may be
//       used to endorse or promote products derived from this software without specific 
//       prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
// SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
// OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
// TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//****************************************************************************************//
#ifndef VARIABLEFACTORY_H
#define VARIABLEFACTORY_H

//	other includes
#include <string>
#include <iostream>
#include <sstream>

#include "Common.h" 
#include "Exception.h" 
#include "LocalVariable.h"
#include "ConstantVariable.h"
#include "ExternalVariable.h"

XERCES_CPP_NAMESPACE_USE

/**
	This class provides a single interface to all variable classes.

*/
class VariableFactory {
public:
	static AbsVariable* GetVariable(std::string varId);

private:
	static DOMElement* variablesElm;
};

/** 
	This class represents an Exception that occured while running the variable factory.
*/
class VariableFactoryException : public Exception {
	public:
		VariableFactoryException(AbsVariable* var = NULL, std::string errMsgIn = "", int severity = ERROR_FATAL, Exception *ex = NULL);
		~VariableFactoryException();

		void SetVariable(AbsVariable* var);
		AbsVariable* GetVariable();

	private:
		AbsVariable* var;
};

#endif
