//
//
//****************************************************************************************//
// Copyright (c) 2002-2011, The MITRE Corporation
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright notice, this list
//       of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright notice, this 
//       list of conditions and the following disclaimer in the documentation and/or other
//       materials provided with the distribution.
//     * Neither the name of The MITRE Corporation nor the names of its contributors may be
//       used to endorse or promote products derived from this software without specific 
//       prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
// SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
// OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
// TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//****************************************************************************************//
#ifndef ENVIRONMENTVARIABLEPROBE_H
#define ENVIRONMENTVARIABLEPROBE_H

#include "AbsProbe.h"

#include <stdlib.h>

#ifdef DARWIN
#include <crt_externs.h>
#define _environ (*_NSGetEnviron())
#else
extern char ** _environ; 
#endif
/**
	This class is responsible for collecting enviroment variable data.
	This class should be platform independent.
*/
class EnvironmentVariableProbe : public AbsProbe {
public:
	virtual ~EnvironmentVariableProbe();
	
	/** Run the environment variable probe */
	virtual ItemVector* CollectItems(Object* object);

	/** Ensure that the EnvironmentVariableProbe is a singleton. */
	static AbsProbe* Instance();
	
private:
	EnvironmentVariableProbe();
    
	/**  */
	virtual Item* CreateItem();

	/** Get all the environment variables currently defined. */
	StringPairVector* GetAllEnvs();

	/** The static instance of the EnvironmentVariableProbe. 
	    All Probes are singletons. The ProbeFactory is responsible for managing instances of Probes. 
    */
	static EnvironmentVariableProbe *instance;
};

#endif
